function [c,err,Upsilon,mu0,U,mu,UB,rho0,r0] = Bur2EKGwUp(r,rho0,r0,Upsilon,state,cini)

%Version 1.0
%Copyright 28 December 2012 by Alan R. Parry
%Permission is granted to all to use this software for non-commercial uses
%only. Permission for commercial use must be obtained in writing from the
%author.
%
%This function computes the "state"^th excited state mass profile
%corresponding to "Upsilon" (in units of inverse light years) that best
%fits a Burkert mass profile of central density "rho0" (in units of solar
%masses per cubic parsec) and core radius "r0" (in units of parsecs).  All
%of which are computed on the radial mesh, "r".  The different fits are
%parameterized along their respective constant Upsilon hyperbolas by the
%value of the half-mass radius, rh (in units of light years).  For
%convenience and consistency in the computations, the input parameters rho0
%and r0 are converted into units of inverse light years squared and light
%years respectively.  We then set rh = c*r0 and test different values of c
%in an organized manner to find the minimum.  The initial value of c can be
%set by the optional input parameter "cini".  If no cini value is
%specified, c is set to 1 by default.
%
%This program works as follows.  First, we create a grid of test rh values
%centered at rh = cini*r0 and of width "step".  We compute the static state
%corresponding to each of these rh values and the corresponding difference
%between their mass profiles and the Burkert profile.  We recenter the grid
%on the value that has the smallest difference.  If the smallest difference
%is already in the center, we cut "step" in half and start the shifting
%process again.  We continue this process until we have the minimum within
%a predetermined error size.
%
%The output parameters are the following.  The parameter "c" is the value
%such that rh = c*r0 is the value of rh that yields the best fitting
%state^th excited state mass profile.  "err" is the approximation of the L2
%norm of the difference between the Burkert profile and the static state
%mass profile computed discretely along the given mesh, r.  "Upsilon" is
%the used value of "Upsilon", computed via the scalings of static states
%and will be extremely close to the set value of the input parameter
%Upsilon only differing due to the inaccuracy of the scalings and our
%computation of the corresponding constants in the scalings.  This value is
%used for consistency with the other subroutines used.  "mu0" is the value
%of mu_0 for the computed best fitting static state.  "U" is a 5 row matrix
%containing the values of the functions f (row 1), p (row 2), V (row 3), M
%(row 4), and muV (row 5) of the best fitting static state.  The functions,
%f, p, V, and M are as defined in the paper, muV is equal to 4pi r^2 mu,
%where mu is the energy density. The energy density "mu" is also an output
%function.  "UB" is a 2 row matrix containing the Burkert energy density
%profile rho (row 1) and the Burkert mass profile M (row 2) computed
%respectively in units of inverse light years squared and light years.
%Finally, the output parameters, "rho0" and "r0", are the Burkert central
%density and core radius converted to units of inverse light years squared
%and light years.

%Define Burkert profile.

[rho,M,rho0,r0] = Burkert(r,rho0,r0);

UB(1,1:length(M)) = rho;
UB(2,1:length(M)) = M;

%Define constants of static states.

Cm = [4.567 10.22 15.81 21.37 26.91 32.42 0 0 0 0 60.32 0 0 0 0 0 0 0 0 0 116.62];
CR = [0.8462 2.2894 3.8253 5.3994 6.9860 8.5606 0 0 0 0 15.1357 0 0 0 0 0 0 0 0 0 29.6822];

%Define grid size, initial center, and step.

s = 3;

Grid = zeros(2,s);

if nargin >=6
    
    cntr = cini;
    
else
    
    cntr = 1;
    
end

step = 1/2;

%Define initial grid.

cp = ceil(s/2);

for k = 1:s    
    
    Grid(1,k) = cntr + (k-cp)*step;
    
end

%Compute the errors for each value of c.

for k = 1:s
    
    c = Grid(1,k);
    
    rh = c*r0;
    
    m = (Cm(state+1)*CR(state+1))/(Upsilon^2*rh);
    
    if rh <= 0
                    
        Grid(2,k) = inf;
                    
    else
    
        [Grid(2,k),~,~,~] = BurEKGdiff(r,M,rh,m,state);
                    
    end
    
end

%Compute the minimum of the errors on this grid.

[~,indx] = min(Grid(2,:));
idx = Grid(1,indx);

%Shift and shrink grid to home in on the minimum.

while step > 2^(-10)
    
    while idx ~= cntr
        
        %Redefine grid and center.
        
        cntr = idx;
        
        Grid1 = zeros(2,s);
        
        for k = 1:s
            
            Grid1(1,k) = cntr + (k-cp)*step;
            
        end
        
        for j = 1:s
            for k = 1:s
                if Grid1(1,j) == Grid(1,k);
                    
                    Grid1(2,j) = Grid(2,k);
                    
                end
            end
        end
        
        %Compute the errors of this new grid.
        
        for k = 1:s
            if Grid1(2,k) == 0
                
                c = Grid1(1,k);
    
                rh = c*r0;
    
                m = (Cm(state+1)*CR(state+1))/(Upsilon^2*rh);
                
                if rh <= 0
                    
                    Grid1(2,k) = inf;
                    
                else
    
                    [Grid1(2,k),~,~,~] = BurEKGdiff(r,M,rh,m,state);
                    
                end
                
            end
        end
        
        Grid = Grid1;
        
        %Compute the minimum of the errors on this grid.
        
        [~,indx] = min(Grid(2,:));
        idx = Grid(1,indx);
        
    end
    
    step = step/2;
    
    %Redefine grid and center.
    
    cntr = idx;
    Grid1 = zeros(2,s);
    
    for k = 1:s
        
        Grid1(1,k) = cntr + (k-cp)*step;
            
    end
    
    Grid1(2,cp) = Grid(2,cp);
    
    %Compute the errors on this new grid.
    
    for k = 1:s
        if k == cp
            
            Grid1(2,k) = Grid(2,k);
            
        else
            
            c = Grid1(1,k);
    
            rh = c*r0;
    
            m = (Cm(state+1)*CR(state+1))/(Upsilon^2*rh);
    
            if rh <= 0
                    
                Grid1(2,k) = inf;
                    
            else
    
                [Grid1(2,k),~,~,~] = BurEKGdiff(r,M,rh,m,state);
                    
            end
            
        end
    end
        
    Grid = Grid1;
        
    %Compute the minimum of the errors on this grid.
    
    [~,indx] = min(Grid(2,:));
    idx = Grid(1,indx);
    
end

%Recompute the minimum function.

c = idx;

rh = c*r0;
m = (Cm(state+1)*CR(state+1))/(Upsilon^2*rh);

[err,Upsilon,mu0,U] = BurEKGdiff(r,M,rh,m,state);

%Generate mesh that U is defined on.

N = length(U)-1;
dr = r(2);
rN = 0:dr:N*dr;

%Recompute c to account for the error in the constants used in the
%CompUpsmu program and make the value of c consistent with the output.

k=1;

while U(4,k) < U(4,N+1)/2
    k = k+1;
end

c = rN(k)/r0;

%Generate the energy density function corresponding to the EKG functions.

mu = Genmu(U,rN,Upsilon,mu0);

%Plot energy density and mass profile comparisons.

K = length(r)-1;
Nuse = min(N,K);

figure(1)
plot(r(1:Nuse+1),M(1:Nuse+1),'--r',r(1:Nuse+1),U(4,1:Nuse+1),'-b')
xlabel('r')
ylabel('Mass')
title('Mass profiles for')


figure(2)
plot(r(1:Nuse+1),rho(1:Nuse+1),'--r',r(1:Nuse+1),mu(1:Nuse+1),'-b')
xlabel('r')
ylabel('Energy Density')
title('Energy density profiles for')

end