function [rho,M,rho0,r0] = Burkert(r,rho0,r0)

%Version 1.0
%Copyright 28 December 2012 by Alan R. Parry
%Permission is granted to all to use this software for non-commercial uses
%only. Permission for commercial use must be obtained in writing from the
%author.
%
%This function computes the Burkert energy density and mass profiles for a
%given value of "rho0" and "r0".  The mesh "r" is given in light years,
%while the rho0 is given in solar masses per cubic parsecs and r0 is given
%in parsecs.  This is to be consistent with the literature.  However, we
%convert everything to years in the output.  So, the output parameters, the
%energy density profile "rho" and the central density "rho0" are given in
%yr^(-2) and the mass profile "M" and core radius "r0" are given in yrs.

%Compute parameters.

N = length(r)-1;

%Define conversion factors.  The first is the number of light years per
%parsec.  The second is the number of light years of mass per solar mass.
%These factors are computed from the definitions used in Binney and
%Tremain's Galactic Dynamics Appendix A, and using geometrized units that
%use the speed of light and the gravitational constant as conversion
%factors between years of time, distance, and mass.

pc2yr = 3.2615017837;
Msol2yr = 1.56075897586e-13;

%Convert rho0 and r0 into year-like units.

rho0 = rho0*Msol2yr/(pc2yr^3);
r0 = r0*pc2yr;

%Create functions.

rho = zeros(1,N+1);
M = zeros(1,N+1);

%Compute function values according to the Burkert profile.

for k = 1:N+1
    
    rho(k) = (rho0*r0^3)/((r(k) + r0)*(r(k)^2 + r0^2));
    M(k) = 2*pi*rho0*r0^3*(log((r(k) + r0)/r0) + log((r(k)^2 + r0^2)/r0^2)/2 - atan(r(k)/r0));
    
end

end