function [Upsilon,mu0,U] = CreateState(dr,rh,m,state)

%Version 1.0
%Copyright 28 December 2012 by Alan R. Parry
%Permission is granted to all to use this software for non-commercial uses
%only. Permission for commercial use must be obtained in writing from the
%author.
%
%This function computes the "state"^th excited state of half mass radius
%"rh" and total mass "m" with discretization interval width dr.
%
%The outputs of this function are the values of "Upsilon" and "mu0"
%corresponding to n = state, total mass m, and half mass radius, rh.  "U"
%is a 5 row matrix containing the values of the functions f (row 1), p (row
%2), V (row 3), M (row 4), and muV (row 5) of the state^th excited state.
%The functions, f, p, V, and M are as defined in the paper, muV is equal to
%4pi r^2 mu, where mu is the energy density.
%
%This function currently only computes the n^th excited state for n in
%{0,1,2,3,4,5,10,20}.

%Define parameters.

CV = [-6.7278429093867012403, -7.5410724296955889450, -7.9315208337699329720, -8.1822933733091037280, -8.3652875084322478190, -8.5086479769048593130, 0, 0, 0, 0, -9.0017685010779546363, 0, 0, 0, 0, 0, 0, 0, 0, 0, -9.5061089019967411383];

%Compute Upsilon and mu0, for the state^th excited state with half mass
%radius rh and mass m.

[Upsilon,mu0] = CompUpsmu(rh,m,state);

%Compute a good range, mesh, and parameters to compute the state^th excited
%state to good accuracy. The values of rmult used here are approximately
%what multiple of rh the current program can compute the n^th excited state
%mass profile to consistently and accurately enough for automation.  Other
%values could possibly work, but these were sufficient for our purposes.
%The first value corresponds to the ground state, the second value to the
%1st excited state, and so on.

rmult = [7, 3.5, 3, 2.5, 2.15, 2, 0, 0, 0, 0, 1.7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.6];

rmax = rmult(state+1)*rh;

N = ceil(rmax/dr);
rmax = N*dr;

%%Here we compute values of V0 sufficiently close to the correct value of V0
%that the procedure can be automated.  Trying to compute test static states
%too far away from the correct value of V0 can crash the program creating
%problems for an automated system.

V0 = CV(state+1)*sqrt(mu0)/Upsilon;
epnt = floor(log10(abs(V0)));

V0a = floor(V0/(10^epnt));
V0b = ceil(V0/(10^epnt));

if V0a == round(V0/(10^epnt))
    
    V0b = V0b - 0.5;
    
else
    
    V0a = V0a + 0.5;
    
end

if 2*V0a == round(2*V0/(10^epnt))
    
    V0b = V0b - 0.25;
    
else
    
    V0a = V0a + 0.25;
    
end

Vlist = [V0a, V0b]*10^epnt;

%Compute the appropriate state.

[~,U,~] = NCEKG_state(1,N,rmax,Upsilon,mu0,state,Vlist);


end