function [V,M,muV] = NCEKG_metric(f,p,r,Upsilon,mu0,lambda)

%Version 1.0
%Copyright 28 December 2012 by Alan R. Parry
%Permission is granted to all to use this software for non-commercial uses
%only. Permission for commercial use must be obtained in writing from the
%author.
%
%This function is actually part of another set of software that evolves the
%spherically symmetric EKG equations through time.  It is only used in one
%place in our current set of software to extend static state mass profiles
%that are too short.  The finite difference techniques used here are
%slightly different than those used in the current program, but it computes
%the same solutions, probably with higher accuracy.
%
%This function computes the compatible metric components M and V for the
%EKG scalar field f.  f and p are the EKG scalar field and its "time
%derivative" (i.e. p = exp(-V)*(1 - 2M/r)^(-1/2)*f_t) respectively defined
%at the radial points in r.  lambda is the reflectivity constant, Upsilon
%is the EKG constant, and mu0 is the scaling parameter in the EKG
%equations.  The output is the potential V, the mass M, and the energy
%density multiplied by the surface area muV (aka M_r).

%Compute parameters.

N = length(r) - 1;
dr = r(2);

%Define functions.

M = zeros(1,N+1);
muV = zeros(1,N+1);
V = zeros(1,N+1);

%Central values of M, V, muV.

M(1) = 0;
V(1) = 0;
muV(1) = 0;

M(2) = 0;

cv1 = (1 - 2*M(2)/r(2));
cv2 = abs(p(2))^2/Upsilon^2 + abs(f(3) - f(1))^2/(4*dr^2*Upsilon^2);
cv3 = 2*real((f(3) - f(1))/(2*dr)*conj(f(2)));
    
cv4a = (f(3) - 2*f(2) + f(1))/dr^2*conj((f(3) - f(1))/(2*dr));
cv4b = (p(3) - p(1))/(2*dr)*conj(p(2));
cv4 = 2/Upsilon^2*real(cv4a + cv4b);
    
Mr = 4*pi*r(2)^2*mu0*(abs(f(2))^2 + cv1*cv2);
    
Mrra = 2*Mr/r(2)*(1 - 4*pi*r(2)^2*mu0*cv2);
Mrrb = 4*pi*r(2)^2*mu0*(cv3 + cv1*cv4 + 2*M(2)/r(2)^2*cv2);
Mrr = Mrra + Mrrb;

muV(2) = Mr;
M(3) = M(2) + Mr*dr + Mrr*dr^2/2;

%To compute Vrr at r=0, we needed to compute Mrrr at r = 0.  We have no
%formulat for either quantity, but the second order centered finite
%difference of Mrrr at r=0 since M is odd is (M(3) - 2*M(2))/dr^2.  So we
%use that to approx Mrrr and in turn use it to approx Vrr at r = 0, which
%lets us get V(2).  From then on we have exact fomulas for Vrr and will use
%those.

Vr = 0;
Vrr = (M(3) - 2*M(2))/(6*dr^2) - 4*pi*mu0*(abs(f(1))^2 - abs(p(1))^2/Upsilon^2);
V(2) = V(1) + Vr*dr + Vrr*dr^2/2;
    
Vr = (M(2)/r(2)^2 - 4*pi*r(2)*mu0*(abs(f(2))^2 - cv1*cv2))/cv1;
    
Vrra = Mr/r(2)*(1/r(2) + 2*Vr - 8*pi*r(2)*mu0*cv2);
Vrrb = Vr/r(2)*(1 - 4*M(2)/r(2)) - 3*M(2)/r(2)^3;
Vrrc = -4*pi*r(2)*mu0*(cv3 - cv1*cv4 - 2*M(2)/r(2)^2*cv2);
Vrr = (1/cv1)*(Vrra + Vrrb + Vrrc);    

V(3) = V(2) + Vr*dr + Vrr*dr^2/2;

%Compute remaining values.

for k = 3:N
    
    cv1 = (1 - 2*M(k)/r(k));
    cv2 = abs(p(k))^2/Upsilon^2 + abs(f(k+1) - f(k-1))^2/(4*dr^2*Upsilon^2);
    cv3 = 2*real((f(k+1) - f(k-1))/(2*dr)*conj(f(k)));
    
    cv4a = (f(k+1) - 2*f(k) + f(k-1))/dr^2*conj((f(k+1) - f(k-1))/(2*dr));
    cv4b = (p(k+1) - p(k-1))/(2*dr)*conj(p(k));
    cv4 = 2/Upsilon^2*real(cv4a + cv4b);
    
    Mr = 4*pi*r(k)^2*mu0*(abs(f(k))^2 + cv1*cv2);
    
    Mrra = 2*Mr/r(k)*(1 - 4*pi*r(k)^2*mu0*cv2);
    Mrrb = 4*pi*r(k)^2*mu0*(cv3 + cv1*cv4 + 2*M(k)/r(k)^2*cv2);
    Mrr = Mrra + Mrrb;
    
    Vr = (M(k)/r(k)^2 - 4*pi*r(k)*mu0*(abs(f(k))^2 - cv1*cv2))/cv1;
    
    Vrra = Mr/r(k)*(1/r(k) + 2*Vr - 8*pi*r(k)*mu0*cv2);
    Vrrb = Vr/r(k)*(1 - 4*M(k)/r(k)) - 3*M(k)/r(k)^3;
    Vrrc = -4*pi*r(k)*mu0*(cv3 - cv1*cv4 - 2*M(k)/r(k)^2*cv2);
    Vrr = (1/cv1)*(Vrra + Vrrb + Vrrc);    
    
    muV(k) = Mr;
    M(k+1) = M(k) + Mr*dr + Mrr*dr^2/2;
    V(k+1) = V(k) + Vr*dr + Vrr*dr^2/2;
    
end

%Adjust V so that it has the appropriate behavior at the boundary.

b = 1/2*log(1 - 2*M(N+1)/r(N+1)) - V(N+1);

V = V + b;

%Compute last value of muV = M_r.

cv1 = (1 - 2*M(N+1)/r(N+1));

if lambda == -1
    
    %We know that p(N+1) = 0 by the BC, but we also used the idea that f is
    %approximately odd about the point (r(N+1),f(N+1)), then f(N+2)-f(N+1)
    %= f(N+1)-f(N) so that f(N+2)-f(N) = 2(f(N+1)-f(N)).  This is also
    %identical to just substituting forward Euler for the centered
    %difference here.
    
    cv2 = abs(f(N+1) - f(N))^2/(dr^2*Upsilon^2);
    muV(N+1) = 4*pi*r(N+1)^2*mu0*(abs(f(N+1))^2 + cv1*cv2);
    
elseif lambda == 1
    
    %f_r(N+1) = 0.
    
    cv2 = abs(p(N+1))^2/Upsilon^2;
    muV(N+1) = 4*pi*r(N+1)^2*mu0*(abs(f(N+1))^2 + cv1*cv2);
    
else
    
    %f_r(N+1) = (lambda-1)/(lambda+1)*p(N+1).
    
    cv2a = (lambda-1)/(lambda+1);
    cv2 = abs(p(N+1))^2/Upsilon^2 + abs(cv2a*p(N+1))^2/Upsilon^2;
    muV(N+1) = 4*pi*r(N+1)^2*mu0*(abs(f(N+1))^2 + cv1*cv2);
    
end
    

end