function [AB1,AB2,V0,omega,Hend,f,V,M,muV] = NCEKG_state_scaled(F0,r,Upsilon,mu0,state,Vlist)

%Version 1.0
%Copyright 28 December 2012 by Alan R. Parry
%Permission is granted to all to use this software for non-commercial uses
%only. Permission for commercial use must be obtained in writing from the
%author.
%
%This function takes the states with small AB1 error and computes the V0
%which yields small AB2 error.  It takes essentially the same input
%parameters as NCEKG_state, only it requests the already generated mesh "r"
%rather than the N and rmax values.
%
%Its output values include both asymptotic behavior values "AB1" and "AB2",
%which should both be very close to 0, the value of "V0" and "omega" for
%the desired static state.  "Hend" is the outermost value of the spatial
%derivative of "f".  "V" is the potential, "M" is the mass profile, and
%"muV" is the energy density times the surface area.
%
%This function was given a version number with Version 2.2.  It is part of
%a subpackage to the NCEKG software.

%Compute N.

N = length(r) - 1;

%Compute the states with small AB1 values (error on the first asymptotic
%behavior we are shooting for) corresponding to the V0 guesses in Vlist.

[~,~,~,~,Va,Ma,~] = NCEKG_state_finder(F0,Vlist(1),r,Upsilon,mu0,state);
[~,~,~,~,Vb,Mb,~] = NCEKG_state_finder(F0,Vlist(2),r,Upsilon,mu0,state);

%Compute the AB2 values (error on the second asymptotic behavior we are
%shooting for in each case.)

AB2a = Va(N+1) - 1/2*log(1 - 2*Ma(N+1)/r(N+1));
AB2b = Vb(N+1) - 1/2*log(1 - 2*Mb(N+1)/r(N+1));

%AB2 appears to be a linear function of V0 or at least approximately linear
%in the region near the zero.  The following computes the zero of that
%line.  This is equivalent to single step of Newton's method.

m = (AB2b - AB2a)/(Vlist(2) - Vlist(1));

V0 = (AB2b - m*Vlist(2))/(-m);

%Compute the state with small AB1 value that corresponds to the computed
%V0 value.

[omegamult,AB1,Hend,f,V,M,muV] = NCEKG_state_finder(F0,V0,r,Upsilon,mu0,state);

%Compute omega and the value of AB2 for the above computed value of V0.

omega = omegamult*Upsilon;
AB2 = V(N+1) - 1/2*log(1 - 2*M(N+1)/r(N+1));

end